@@ -3,9 +3,13 @@ import os
 # RASA CODELESS CLI
 PACKAGE_NAME = "rasac"
 PACKAGE_NAME_PYPI = "rasac"
-PACKAGE_VERSION = "1.0.0"
+PACKAGE_VERSION = "2.0.0"
 PACKAGE_VERSION_LONG = f'RASAC Version: {PACKAGE_VERSION}'
 LANGUAGES_SUPPORTED = ['en', 'si']
+LOGGING_FORMAT_STR = '%(asctime)s\t%(levelname)s\t%(name)s - %(message)s'
+DOTENV_FILES = [".env"]
+PACKAGE_README_PYPI = "READMEPyPI.md"
+PACKAGE_REQUIREMENTS = "requirements.txt"
 
 
 class InterfaceType:
@@ -54,6 +58,7 @@ DEFAULT_INIT_DEST_DIR_NAME = "./"
 RASA_DIRS_IN_RASAC_INIT = ["tensorboard"]
 INVALID_DIR_NAME_CHARS = ['\\', '/', '<', '>', ':', '*', '?', '|']
 ALLOWED_INIT_DIR_NAMES = [".", "./", "None", "none"]
+RASA_CODELESS_PROJECT_DIRS = ["bot_store", "rasac_cache", "tensorboard"]
 
 
 class DestinationDirType:
@@ -65,18 +70,20 @@ class DestinationDirType:
 
 
 # RASA
-DEFAULT_RASA_CONFIG_PATH = "./config.yml"
+DEFAULT_RASA_CONFIG_PATH = "config.yml"
 YAML_EXTENSIONS = [".yaml", ".yml"]
 DEFAULT_YAML_VERSION = "2.0"
 DEFAULT_RASA_ROOT_DIR = "./"
 DEFAULT_VERSION_YAML_TAG = "version"
 DEFAULT_PIPELINE_YAML_TAG = "pipeline"
-DEFAULT_MODEL_PATH = "./models"
+DEFAULT_MODEL_PATH = "models"
 RASA_MODEL_EXTENSIONS = [".tar.gz"]
-RASA_288_MODEL_REGEX = "^(\\d{8})\\-(\\d{6}).tar.gz$"
+RASA_MODEL_REGEX = "^(\\d{8})\\-(\\d{6}).tar.gz$"
+RASA_BOTSTORE_DIR_REGEX = "^(\\d{8})\\-(\\d{6})$"
+RASA_MODEL_TIMESTAMP_PATTERN = "%Y%m%d-%H%M%S.tar.gz"
 NLU_FALLBACK_TAG = "nlu_fallback"
-DEFAULT_CACHE_PATH = "./"
-DEFAULT_DATA_PATH = "./data"
+DEFAULT_CACHE_PATH = DEFAULT_RASA_ROOT_DIR
+DEFAULT_DATA_PATH = "data"
 DEFAULT_NLU_YAML_TAG = "nlu"
 DEFAULT_NLU_YAML_VERSION = "2.0"
 DEFAULT_NLU_INTENT_TAG = "intent"
@@ -89,7 +96,10 @@ DEFAULT_LATEST_TAG = "latest"
 class FilePermission:
     READ = "r"
     WRITE = "w"
-    OVERWRITE = "w+"
+    APPEND = "a"
+    WRITE_PLUS = "w+"
+    READ_PLUS = "r+"
+    APPEND_PLUS = "a+"
 
 
 class Encoding:
@@ -142,7 +152,7 @@ class TermColor:
 
 # CONFIGS
 class Config:
-    DEFAULT_CONFIG_PATH = "./rasac_config.yml"
+    DEFAULT_CONFIG_PATH = "rasac_config.yml"
     BASE_CONFIGS_KEY = "rasac_base_configs"
     SERVER_CONFIGS_KEY = "rasac_server_configs"
     CONFIG_PATH_KEY = "rasa_config_path"
@@ -166,6 +176,12 @@ class TensorboardDirectories:
     VALIDATION = "validation"
 
 
+class TensorboardMetrics:
+    ACCURACY = "accuracy"
+    LOSS = "loss"
+    F1 = "f1-score"
+
+
 MODEL_ID_TAG = "model_id"
 EPOCHS_TAG = "epochs"
 TRAIN_ACCURACY_TAG = "train"
@@ -173,7 +189,7 @@ VALIDATION_ACCURACY_TAG = "test"
 TRAIN_LOSS_TAG = "train_loss"
 VALIDATION_LOSS_TAG = "test_loss"
 CONFIG_TAG = "config"
-DEFAULT_TENSORBOARD_LOGDIR = "./tensorboard"
+DEFAULT_TENSORBOARD_LOGDIR = "tensorboard"
 TENSORBOARD_INTENT_ACCURACY_TAG = "epoch_i_acc"
 TENSORBOARD_INTENT_LOSS_TAG = "epoch_t_loss"
 TENSORBOARD_SIMPLE_VALUE_TAG = "simple_value"
@@ -182,5 +198,54 @@ TENSORBOARD_RESULTS_FILE_EXTENSION = "*.v2"
 
 # TRAINING QUEUE
 PROCESS_ID_NONE = -99
-TRAINING_QUEUE = os.path.join("./rasac_cache", "training_queue.db")
+TRAINING_QUEUE = os.path.join("rasac_cache", "training_queue.db")
 TRAINING_QUEUE_TABLE = "training_queue"
+
+
+# BOTSTORE
+BOTSTORE_PATH = "bot_store"
+BOTSTORE_ASSETS = {
+    "duplicate": [
+        "actions",
+        "custom",
+        "data",
+        "kolloqe_components",
+        "seetm_components",
+        "seetm_eval",
+        "seetm_exports",
+        "seetm_maps",
+        "tests",
+        ".dockerignore",
+        ".env",
+        "config.yml",
+        "config-dev.yml",
+        "credentials.yml",
+        "credentials-dev.yml",
+        "credentials-docker.yml",
+        "dime-config.yml",
+        "domain.yml",
+        "endpoints.yml",
+        "endpoints-dev.yml",
+        "endpoints-docker.yml",
+        "requirements.txt",
+        "requirements.txt",
+        "seetm-config.yml",
+    ],
+    "move_dir": [
+        "results",
+    ],
+    "move_dir_content": [
+        "tensorboard",
+    ],
+}
+
+
+class AssetType:
+    DUPLICATE = "duplicate"
+    MOVE_DIR = "move_dir"
+    MOVE_DIR_CONTENT = "move_dir_content"
+
+
+class SourceType:
+    FILE = "file"
+    DIRECTORY = "dir"
