@@ -7,13 +7,13 @@ import sys
 from collections import OrderedDict as OrderedDictColl
 from datetime import datetime
 from os import path
-from typing import List, Optional, Text, OrderedDict, Dict, NoReturn, Union, Tuple
+from typing import List, Optional, Text, OrderedDict, Dict, NoReturn, Union, Tuple, Any
 from uuid import uuid4
 
-from rasa.shared.data import get_data_files, is_nlu_file
-from ruamel import yaml as yaml
-from ruamel.yaml.error import YAMLError
-
+from rasa.shared.data import (
+    get_data_files,
+    is_nlu_file,
+)
 from rasa_codeless.shared.constants import (
     DEFAULT_CACHE_PATH,
     DEFAULT_DATA_PATH,
@@ -27,10 +27,18 @@ from rasa_codeless.shared.constants import (
     DEFAULT_NLU_EXAMPLES_TAG,
     TermColor,
     RASA_MODEL_EXTENSIONS,
-    RASA_288_MODEL_REGEX,
+    RASA_MODEL_REGEX,
     DEFAULT_CASE_SENSITIVE_MODE,
     DEFAULT_LATEST_TAG,
-    DEFAULT_INIT_DEST_DIR_NAME, DEFAULT_RASA_ROOT_DIR, DEFAULT_TENSORBOARD_LOGDIR, DEFAULT_MODEL_PATH,
+    DEFAULT_INIT_DEST_DIR_NAME,
+    DEFAULT_RASA_ROOT_DIR,
+    DEFAULT_TENSORBOARD_LOGDIR,
+    DEFAULT_MODEL_PATH,
+    BOTSTORE_PATH,
+    RASA_BOTSTORE_DIR_REGEX,
+    BOTSTORE_ASSETS,
+    AssetType,
+    SourceType,
 )
 from rasa_codeless.shared.exceptions.io import (
     YAMLFormatException,
@@ -38,10 +46,48 @@ from rasa_codeless.shared.exceptions.io import (
     ModelNotFoundException,
     InvalidFileExtensionException,
     FileSizeInspectingException,
+    CreateDirectoryException,
+    DirectoryPathNotSpecifiedException,
+    InvalidDirectoryPathException,
+    ModelPersistException,
+    InvalidAssetTypeException,
 )
-from rasa_codeless.shared.exceptions.server import InvalidDirectoryPathException
+from ruamel import yaml as yaml
+from ruamel.yaml.error import YAMLError
 
 logger = logging.getLogger(__name__)
+yml = yaml.YAML()
+yml.indent(mapping=2, sequence=4, offset=2)
+
+
+def create_dir(dir_path: Text, force: bool = False) -> NoReturn:
+    """
+    Creates the DIME caching directory to
+    store the data/ model fingerprints and
+    calculated global feature importance
+    scores
+
+    Args:
+        dir_path: directory path where the dir
+            should be created
+        force: whether to delete and create an
+            existing dir or not
+    """
+
+    if not dir_path:
+        logger.error("Directory name is not specified. rasac will "
+                     "terminate the request.")
+        raise DirectoryPathNotSpecifiedException()
+
+    if not dir_exists(dir_path):
+        os.mkdir(dir_path)
+    else:
+        if force:
+            os.rmdir(dir_path)
+            os.mkdir(dir_path)
+
+    if not dir_exists(dir_path):
+        raise CreateDirectoryException()
 
 
 def _create_cache_dir(cache_data_dir: Text = DEFAULT_CACHE_PATH) -> bool:
@@ -70,36 +116,6 @@ def _create_cache_dir(cache_data_dir: Text = DEFAULT_CACHE_PATH) -> bool:
     return dir_exists(cache_data_dir)
 
 
-def _get_dir_file_list(
-        dir_path: Text = DEFAULT_DATA_PATH,
-        file_suffixes: List = YAML_EXTENSIONS
-) -> Optional[List]:
-    """
-    Returns the list files available in a given
-    directory path and its subdirectories. able
-    to check extensions if the extensions are
-    provided as a list
-
-    Args:
-        dir_path: path where to read the list of directories
-        file_suffixes: list of file extensions to look for. if
-            not provided, the method will output all the files
-            available in the given destination directory
-
-    Returns:
-        list of files available, or None
-    """
-
-    file_paths = list()
-
-    for (dir_path, dir_names, file_names) in os.walk(dir_path):
-        if file_suffixes:
-            file_paths += [path.join(dir_path, file) for file in file_names if str(file).endswith(tuple(file_suffixes))]
-        else:
-            file_paths += [path.join(dir_path, file) for file in file_names]
-    return file_paths
-
-
 def get_all_existing_file_list(
         dir_path: Text = DEFAULT_DATA_PATH,
 ) -> Optional[List]:
@@ -445,7 +461,7 @@ def get_rasa_testing_data(
                 raise InvalidFileExtensionException("The specified file is not a valid "
                                                     "YAML file")
         else:
-            testing_data_files = _get_dir_file_list(dir_path=testing_data_dir, file_suffixes=file_ext)
+            testing_data_files = get_dir_file_list(dir_path=testing_data_dir, file_suffixes=file_ext)
         logger.info(f"{len(testing_data_files)} YAML files were found in the testing data directory.")
 
         if testing_data_files:
@@ -505,7 +521,7 @@ def set_cli_color(text_content: Text = None, color: str = TermColor.NONE_C):
     return color + str(text_content) + TermColor.END_C
 
 
-def get_latest_model_name(models_path: Text) -> Text:
+def get_latest_model_name(models_path: Union[Text, List]) -> Text:
     """
     Finds all RASA models available in the model dir
     provided and selects the latest model using the
@@ -519,21 +535,24 @@ def get_latest_model_name(models_path: Text) -> Text:
     Returns:
         latest model name as a string
     """
+    if isinstance(models_path, Text):
+        model_list = get_dir_file_list(
+            dir_path=models_path,
+            file_suffixes=RASA_MODEL_EXTENSIONS,
+        )
+        if not model_list:
+            raise ModelNotFoundException(f"Could not find a compatible RASA model in the specified "
+                                         f"location. Make sure there is a valid RASA model in "
+                                         f"`{models_path}`")
 
-    model_list = _get_dir_file_list(
-        dir_path=models_path,
-        file_suffixes=RASA_MODEL_EXTENSIONS,
-    )
-    if not model_list:
-        raise ModelNotFoundException(f"Could not find a compatible RASA model in the specified "
-                                     f"location. Make sure there is a valid RASA model in "
-                                     f"`{models_path}`")
+    else:
+        model_list = models_path
 
     models_dict = {
         os.path.split(model_path)[-1]:
-            int(''.join(list(re.findall(RASA_288_MODEL_REGEX, os.path.split(model_path)[-1])[0])))
+            int(''.join(list(re.findall(RASA_MODEL_REGEX, os.path.split(model_path)[-1])[0])))
         for model_path in model_list
-        if re.findall(RASA_288_MODEL_REGEX, os.path.split(model_path)[-1])
+        if re.findall(RASA_MODEL_REGEX, os.path.split(model_path)[-1])
     }
 
     latest_model = None
@@ -589,7 +608,7 @@ def rasa_models_dir_exists(dir_path: Text) -> bool:
     if not os.path.exists(dir_path):
         return False
 
-    models = _get_dir_file_list(dir_path=dir_path, file_suffixes=RASA_MODEL_EXTENSIONS)
+    models = get_dir_file_list(dir_path=dir_path, file_suffixes=RASA_MODEL_EXTENSIONS)
     if len(models) == 0:
         return False
 
@@ -600,7 +619,7 @@ def rasa_model_exists(models_path: Text, model_name: Text) -> bool:
     if str(model_name).lower() == DEFAULT_LATEST_TAG:
         return True
 
-    if not re.fullmatch(RASA_288_MODEL_REGEX, model_name):
+    if not re.fullmatch(RASA_MODEL_REGEX, model_name):
         return False
 
     if file_exists(file_path=os.path.join(models_path, model_name)):
@@ -655,5 +674,77 @@ def dir_cleanup(dir_path: Text):
 
 
 def get_model_list(model_path=DEFAULT_MODEL_PATH, file_suffixes=RASA_MODEL_EXTENSIONS) -> List:
-    file_list = get_dir_file_list(dir_path=DEFAULT_MODEL_PATH, file_suffixes=RASA_MODEL_EXTENSIONS)
+    file_list = get_dir_file_list(dir_path=model_path, file_suffixes=file_suffixes)
     return [os.path.split(model_name)[-1] for model_name in file_list]
+
+
+def get_botstore_model_list(
+        botstore_path: Text = BOTSTORE_PATH,
+        pattern: Text = RASA_BOTSTORE_DIR_REGEX
+) -> List:
+    top_level_assets = get_existing_toplevel_file_list(dir_path=botstore_path)
+    valid_model_dirs = [
+        dir_+RASA_MODEL_EXTENSIONS[0] for dir_ in top_level_assets if re.match(pattern, str(dir_))] \
+        if top_level_assets else []
+    return valid_model_dirs
+
+
+def get_valid_model_list(
+        botstore_path: Text = BOTSTORE_PATH,
+        models_path: Text = DEFAULT_MODEL_PATH,
+) -> List:
+    botstore_model_list = get_botstore_model_list(botstore_path=botstore_path)
+    model_list = get_model_list(model_path=models_path)
+    valid_model_list = list(set(model_list).intersection(set(botstore_model_list)))
+    return valid_model_list
+
+
+def write_yaml_file(
+        yaml_file: Text,
+        yaml_content: Any,
+        mode: Text = FilePermission.WRITE_PLUS,
+) -> NoReturn:
+    with open(yaml_file, mode) as yaml_file_:
+        yml.dump(yaml_content, yaml_file_)
+
+
+def persist_model_data(
+        model_name: Text,
+        botstore_path: Text = BOTSTORE_PATH,
+        assets: Dict = None
+) -> NoReturn:
+    try:
+        model_timestamp = model_name.replace(RASA_MODEL_EXTENSIONS[0], "")
+        os.makedirs(os.path.join(botstore_path, model_timestamp), exist_ok=True)
+        assets_to_persist = BOTSTORE_ASSETS
+
+        if assets:
+            for asset_type in assets.keys():
+                if asset_type in assets_to_persist:
+                    assets_to_persist[asset_type] += assets[asset_type]
+
+        for asset_type, assets_paths in assets_to_persist.items():
+            for path_ in assets_paths:
+                src_ = path_
+                dst_ = os.path.join(botstore_path, model_timestamp, path_)
+                if not os.path.exists(path=src_):
+                    continue
+
+                src_type = SourceType.DIRECTORY if os.path.isdir(src_) else SourceType.FILE
+
+                if asset_type == AssetType.DUPLICATE:
+                    shutil.copytree(src=src_, dst=dst_, dirs_exist_ok=True) \
+                        if src_type == SourceType.DIRECTORY \
+                        else shutil.copyfile(src=src_, dst=dst_)
+                elif asset_type == AssetType.MOVE_DIR:
+                    shutil.move(src=src_, dst=dst_)
+                elif asset_type == AssetType.MOVE_DIR_CONTENT:
+                    dir_content = get_existing_toplevel_file_list(src_)
+                    for content in dir_content:
+                        shutil.move(os.path.join(src_, content), os.path.join(dst_, content))
+                else:
+                    raise InvalidAssetTypeException()
+    except InvalidAssetTypeException as e:
+        raise InvalidAssetTypeException(e)
+    except Exception as e:
+        raise ModelPersistException(e)
